;;;; debug.jl -- Lisp debugger (well, single-stepper anyway)
;;;  Copyright (C) 1993, 1994 John Harper <jsh@ukc.ac.uk>

;;; This file is part of Jade.

;;; Jade is free software; you can redistribute it and/or modify it
;;; under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.

;;; Jade is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with Jade; see the file COPYING.  If not, write to
;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

(provide 'debug)

(setq
  debug-buffer (make-buffer "*debugger*")
  debug-keymap (make-keylist))
(set-buffer-special debug-buffer t)
(add-buffer debug-buffer)

(bind-keys debug-keymap
  "s" 'debug-step
  "i" '(debug-set-result nil)
  "n" 'debug-next
  "c" 'debug-continue
  "b" '(debug-backtrace 2)
  "esc" '(debug-set-result (eval (read (prompt "Eval: ")))))

(with-buffer debug-buffer
  (setq ctrl-c-keymap debug-keymap)
  (split-line)
  (insert "::Lisp Debugger::\n
ctrl-c s   : step into form
ctrl-c i   : ignore form
ctrl-c n   : continue until next form
ctrl-c c   : continue
ctrl-c b   : print backtrace
ctrl-c esc : set value of form\n\n"))

;;;###autoload
(defun debug-entry (debug-obj debug-depth)
  (with-buffer debug-buffer
    (goto-char (line-start (file-end)))
    (format debug-buffer "%s%S\n" (make-string (* 2 debug-depth)) debug-obj)
    (goto-glyph (next-line 1 (indent-pos (prev-line))))
    (catch 'debug
      (recursive-edit))))

(defun debug-exit (debug-val debug-depth)
  (with-buffer debug-buffer
    (goto-char (line-start (file-end)))
    (format debug-buffer "%s=> %S\n" (make-string (* 2 debug-depth)) debug-val)))

;;;###autoload
(defun debug-error-entry (error-list)
  (with-buffer debug-buffer
    (goto-char (line-start (file-end)))
    (format debug-buffer "*** Error: %s: %S\n" (unless (get (car error-list) 'error-message) (car error-list)) (cdr error-list))
    (catch 'debug
      (recursive-edit)
      nil)))

(defun debug-step ()
  (if (boundp 'debug-obj)
      (throw 'debug (cons 1 debug-obj))
    (beep)))

(defun debug-set-result (value)
  (if (boundp 'debug-obj)
      (throw 'debug (cons 4 value))
    (beep)))

(defun debug-next ()
  (if (boundp 'debug-obj)
      (throw 'debug (cons 2 debug-obj))
    (beep)))

(defun debug-continue ()
  (cond
    ((boundp 'debug-obj)
      (throw 'debug (cons 3 debug-obj)))
    ((boundp 'error-list)
      (throw 'debug))
    (t
      (beep))))

;; DEPTH is the number of stack frames to discard
(defun debug-backtrace (depth)
  (goto-char (line-start (file-end)))
  (let
      ((old-pos (cursor-pos)))
    (backtrace debug-buffer)
    (delete-area old-pos (next-line (1+ depth) (copy-pos old-pos)))
    (split-line)))
