;;;; find-autoloads.jl -- Rebuild the autoload.jl file
;;;  Copyright (C) 1993, 1994 John Harper <jsh@ukc.ac.uk>

;;; This file is part of Jade.

;;; Jade is free software; you can redistribute it and/or modify it
;;; under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.

;;; Jade is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with Jade; see the file COPYING.  If not, write to
;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

(provide 'find-autoloads)

;;; Code to manipulate the sequence of autoload functions in autoload.jl
;;; Basically you just do `ESC x add-autoloads' to add all marked functions
;;; and macros to the list and `ESC x remove-autoloads' to take them out.

(defun autoload-insert-entry (str)
  (if (find-next-string str (file-start))
      (progn
	(delete-area find-last-start-pos (line-end find-last-start-pos))
	(goto-char find-last-start-pos))
    (unless (find-prev-regexp "^;;; ::autoload-end::" (file-end))
      (error "autoload.jl file malformed"))
    (goto-char (line-end (prev-line 1 find-last-start-pos)))
    (insert "\n"))
  (insert str))

(defun autoload-remove-entry (str)
  (when (find-next-string str (file-start))
    (goto-char find-last-start-pos)
    (delete-area find-last-start-pos
		 (line-start (next-line 1 find-last-start-pos)))))

(defun autoload-do-magic (buf line-fun)
  (let
      (aload-buf)
    (when (setq aload-buf (open-file (file-concat lisp-lib-dir "autoload.jl")))
      (goto-buffer aload-buf)
      (let
	  ((pos (file-start))
	   form
	   (short-file-name (regexp-expand "^(.+)\\.jl$"
					   (base-name (file-name buf))
					   "\\1"))
	   (count 0))
	(while (setq pos (find-next-regexp "^;;;###autoload" pos buf))
	  (setq form (regexp-expand-line "^;;;###autoload[\t ]*(.*)$" "\\1"
					  pos buf))
	  (when (and form (not (equal "" form)))
	    (funcall line-fun form)
	    (setq count (1+ count))
	    (title-now form))
	  (next-line 1 pos)
	  (when (and (regexp-match-line "^\\(def(macro|un) " pos buf)
		     (setq form (read (cons buf pos)))
		     (memq (car form) '(defun defmacro)))
	    (setq form (format-to-string "(autoload '%s %S)" (nth 1 form)
					 short-file-name))
	    (when (funcall line-fun form)
	      (setq count (1+ count))
	      (title-now form))))
	count))))

;;;###autoload
(defun add-autoloads (&optional buf)
  "(add-autoloads [BUFFER])
Add all functions, macros or variables in the BUFFER marked by the magic rune
`;;;###autoload' to the `autoload.jl' file."
  (title (format-to-string "Found %d autoload definition(s)"
			   (autoload-do-magic (unless buf (current-buffer))
					      #'autoload-insert-entry))))

;;;###autoload
(defun remove-autoloads (&optional buf)
  "(remove-autoloads [BUFFER])
Removes all autoload definitions in the buffer BUFFER which are marked by the
string `;;;###autoload' from the `autoload.jl' file."
  (title (format-to-string "Removed %d autoload definition(s)"
			   (autoload-do-magic (unless buf (current-buffer))
					      #'autoload-remove-entry))))
