;;;; loadkeys.jl -- Set up standard keybindings
;;;  Copyright (C) 1993, 1994 John Harper <jsh@ukc.ac.uk>

;;; This file is part of Jade.

;;; Jade is free software; you can redistribute it and/or modify it
;;; under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.

;;; Jade is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with Jade; see the file COPYING.  If not, write to
;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

(setq
  global-keymap (make-keytab)
  ctrl-x-keymap (make-keytab)
  ctrl-x-4-keymap (make-keylist)
  ctrl-x-5-keymap (make-keylist)
  esc-keymap (make-keytab)
  ctrl-c-keymap nil
  unbound-key-hook nil
  keymap-path (list global-keymap)
  close-gadget-hook (cons 'close-window nil)
  meta-sends-esc t)

(set-buffer-variable 'keymap-path)
(set-buffer-variable 'buffer-keymap)
(set-buffer-variable 'ctrl-c-keymap)
(set-buffer-variable 'unbound-key-hook)

(setq
  mark-1 (make-mark)
  mark-2 (make-mark)
  mark-3 (make-mark))

(defun self-insert ()
  "(self-insert)
This function can be bound to a key to make it insert its usual character
sequence. This is usually used to ``hide'' a previously bound definition of
the key."
  (insert (current-key-string)))

(bind-keys global-keymap
  "UP"                'goto-prev-line
  "DOWN"              'goto-next-line
  "LEFT"              'goto-left-char
  "RIGHT"             'goto-right-char
  "Shift-UP"          '(progn (set-auto-mark) (goto-char (pos nil 0)))
  "Shift-DOWN"        '(progn (set-auto-mark) (goto-char (pos nil (1- (file-length)))))
  "Shift-LEFT"        'goto-line-start
  "Shift-RIGHT"       'goto-line-end
  "Ctrl-UP"           'prev-screen
  "Ctrl-DOWN"         'next-screen
  "Ctrl-LEFT"         '(goto-char (left-char 40))
  "Ctrl-RIGHT"        '(goto-char (right-char 40))
  "Ctrl-TAB"          'goto-next-tab
  "Shift-TAB"         'goto-prev-tab
  "RET"               'split-line
  "BACKSPACE"         'backspace-char
  "DEL"               'delete-char
  "Shift-BACKSPACE"   '(write-clip 0 (cut-area (line-start) (cursor-pos)))
  "Shift-DEL"         '(write-clip 0 (cut-area (cursor-pos) (line-end)))
  "Ctrl-DEL"          '(write-clip 0 (cut-area (line-start) (line-start (next-line))))
  "Ctrl-@"            'set-auto-mark
  "Ctrl-a"            'goto-line-start
  "Ctrl-b"            'goto-prev-char
  "Ctrl-c"            '(setq next-keymap-path '(ctrl-c-keymap))
  "Ctrl-d"            'delete-char
  "Ctrl-e"            'goto-line-end
  "Ctrl-f"            'goto-next-char
  "Ctrl-i"            'insert-block
  "Ctrl-j"            'goto-line
  "Ctrl-k"            'kill-line
  "Ctrl-m"            'block-toggle
  "Ctrl-M"            'toggle-rect-blocks
  "Ctrl-n"            'goto-next-line
  "Ctrl-o"            '(progn (split-line) (goto-prev-char))
  "Ctrl-p"            'goto-prev-line
  "Ctrl-q"            '(setq next-keymap-path t)
  "Ctrl-u"            'add-command-arg
  "Ctrl-U"            'clear-command-args
  "Ctrl-v"            'next-screen
  "Ctrl-w"            '(write-clip 0 (cut-block))
  "Ctrl-x"            '(setq next-keymap-path '(ctrl-x-keymap))
  "Ctrl-y"            '(insert (read-clip 0))
  "Ctrl-Y"            '(insert-rect (read-clip 0))
  "Ctrl-z"            'delete-block
  "Ctrl-."            'rotate-buffers-forward
  "Ctrl-,"            'rotate-buffers-backward
  "Ctrl-SPACE"        'block-toggle
  "Ctrl-]"            'abort-recursive-edit
  "ESC"               '(setq next-keymap-path '(esc-keymap))
  "F1"                '(goto-mark mark-1)
  "F2"                '(goto-mark mark-2)
  "F3"                '(goto-mark mark-3)
  "Shift-F1"          '(set-mark mark-1 (cursor-pos) (current-buffer))
  "Shift-F2"          '(set-mark mark-2 (cursor-pos) (current-buffer))
  "Shift-F3"          '(set-mark mark-3 (cursor-pos) (current-buffer))
  "LMB-CLICK1"        '(goto-char (mouse-pos))
  "LMB-CLICK2"        'block-toggle
  "LMB-MOVE"          '(goto-char (mouse-pos))
  "RMB-CLICK1"        '(if (window-asleep-p) (unsleep-window) (sleep-window))
  "Shift-LMB-CLICK1"  'block-kill
)
(bind-keys ctrl-x-keymap
  "Ctrl-c"            'save-and-quit
  "Ctrl-f"            'find-file
  "Ctrl-l"            'downcase-block
  "Ctrl-r"            'find-file-read-only
  "Ctrl-s"            'save-file
  "Ctrl-u"            'upcase-block
  "Ctrl-v"            'find-alternate-file
  "Ctrl-w"            'save-file-as
  "Ctrl-x"            'swap-cursor-and-auto-mark
  "0"                 'close-window
  "1"                 'close-other-windows
  "2"                 'open-window
  "4"                 '(setq next-keymap-path '(ctrl-x-4-keymap))
  "5"                 '(setq next-keymap-path '(ctrl-x-5-keymap))
  "b"                 'switch-to-buffer
  "f"                 'set-fill-column
  "h"                 '(progn (block-kill) (block-start (file-start)) (block-end (file-end)))
  "i"                 'insert-file
  "k"                 'kill-buffer
  "o"                 '(set-current-window (next-window) t)
  "q"                 '(set-buffer-read-only nil (not (buffer-read-only-p)))
  "s"                 'save-some-buffers
  "`"                 'next-error
  "#"                 'server-close-file
)
(bind-keys ctrl-x-4-keymap
  "Ctrl-f"            '(in-other-window (find-file))
  "a"                 '(in-other-window (add-change-log-entry))
  "b"                 '(in-other-window (switch-to-buffer))
  "f"                 '(in-other-window (find-file))
  "h"                 '(in-other-window (help) (help-help))
  "i"                 '(in-other-window (info))
  "`"                 '(in-other-window (next-error))
)
(bind-keys ctrl-x-5-keymap
  "Ctrl-f"            '(in-new-window (find-file))
  "a"                 '(in-new-window (add-change-log-entry))
  "b"                 '(in-new-window (switch-to-buffer))
  "f"                 '(in-new-window (find-file))
  "h"                 '(in-new-window (help) (help-help))
  "i"                 '(in-new-window (info))
  "`"                 '(in-new-window (next-error))
)
(bind-keys esc-keymap
  "ESC"		      '(prin1 (eval (read (cons 0 (prompt "Eval: ")))) t)
  "LEFT"              '(goto-char (backward-word))
  "RIGHT"             '(goto-char (forward-word))
  "UP"                '(goto-char (prev-paragraph))
  "DOWN"              '(goto-char (next-paragraph))
  "Ctrl-c"            '(throw 'exit nil)
  "!"                 '(system (prompt "System: "))
  "<"                 '(progn (set-auto-mark) (goto-file-start))
  ">"                 '(progn (set-auto-mark) (goto-file-end))
  "@"                 'mark-word
  "a"                 'add-change-log-entry
  "b"                 '(goto-char (backward-word))
  "c"                 'capitalize-word
  "d"                 '(write-clip 0 (cut-area (cursor-pos) (forward-word)))
  "f"                 '(goto-char (forward-word))
  "h"                 'mark-paragraph
  "i"                 '(insert "\t")
  "l"                 'downcase-word
  "m"                 '(goto-glyph (indent-pos))
  "n"                 '(progn (set-auto-mark) (goto-char (match-brackets)))
  "p"                 '(simple-replace nil)
  "P"                 '(simple-replace t)
  "u"                 'upcase-word
  "v"                 'prev-screen
  "w"                 '(write-clip 0 (copy-block))
  "x"                 'command-prompt
  "["                 '(goto-char (prev-paragraph))
  "]"                 '(goto-char (next-paragraph))
  ";"		      'insert-comment
  "~"                 '(set-buffer-modified nil nil)
  "TAB"               'goto-next-tab
  "DEL"               '(write-clip 0 (cut-area (cursor-pos) (forward-word)))
  "BACKSPACE"         '(write-clip 0 (cut-area (backward-word) (cursor-pos)))
  "HELP"              '(if (window-asleep-p) (unsleep-window) (sleep-window))
)
